//NumbersSortedByNumberOfBits.c - Alain Brobecker
//2015/08/06: first version
//2019/05/24: added double array for nCp occurrence
//            ( pgdc & nCk routines are from 2010/02/12 )
#include <stdio.h>  // file functions, etc...
#include <stdlib.h> // exit(), malloc()

#define MaxNbBits 7
#define MaxNbBits_Power2 128

int NbValuesWithNBits[MaxNbBits+1];
int RulesUsed[MaxNbBits_Power2],NbBitsSet[MaxNbBits_Power2];

//Returns greatest common divisor of two numbers
int gcd(p,q) int p,q; {
  int r;
  if((r=p%q)==0) {
    return q;
  } else {
    return(gcd(q,r));
  }
}

//                               n*(n-1)*...*(k+1)
//Returns nCk=n!/(k!*(n-k)!) = ---------------------
//                              (n-k)*(n-k-1)*...*1
//But n*(n-1)*...*(k+1)=(n-k+k)*...*(1+k)  also has n-k factors.
int nCk(int N, int K) {
  int t,p,q,d;
  p=1;
  q=1;
  for(t=0;t<N-K;t++) {
    p=p*(N-t);
    q=q*(t+1);
    d=gcd(p,q);
    p=p/d;
    q=q/d;
//    printf("%d/%d\n",p,q);
  }
  return(p); //we know that p/q is an integer
}


//*********** **
//****      ****
//**** MAIN ****
//****      ****
//*********** **
int main(argc,argv) int argc; char *argv[]; {
  setbuf(stdout, NULL); // no buffering for output!

  int c,d,t,minpos,minnb,minval;
  int NbBits,position;

  for(c=0;c<=MaxNbBits;c++) {
    NbValuesWithNBits[c]=0;
  }

  //count the number of bits set
  for(c=0;c<MaxNbBits_Power2;c++) {
    RulesUsed[c]=c;
    NbBits=0;
    for(d=0;d<MaxNbBits;d++) {
      if((c & (1<<d))!=0) { NbBits++; }
    }
    NbBitsSet[c]=NbBits;
    NbValuesWithNBits[NbBits]=NbValuesWithNBits[NbBits]+1;
  }

  // sort by inCkeasing number of bits sets
  for(c=0;c<MaxNbBits_Power2-1;c++) {
    minnb=NbBitsSet[c];
    minpos=c;
    minval=RulesUsed[c];
    for(d=c+1;d<MaxNbBits_Power2;d++) {
      if((NbBitsSet[d]<minnb) || ((NbBitsSet[d]==minnb) && (RulesUsed[d]<minval))) {
//      if(NbBitsSet[d]<=minnb) {
        minnb=NbBitsSet[d];
        minpos=d;
        minval=RulesUsed[d];
      }
    }
    //exchange
    t=NbBitsSet[c];
    NbBitsSet[c]=NbBitsSet[minpos];
    NbBitsSet[minpos]=t;
    t=RulesUsed[c];
    RulesUsed[c]=RulesUsed[minpos];
    RulesUsed[minpos]=t;
  }

  //print results
  printf("#define MaxNbBits %d\n",MaxNbBits);
  printf("#define MaxNbBits_Power2 %d\n",1<<MaxNbBits);
  //start position
  printf("int ValuesWithKBits_StartPosition[MaxNbBits+1]={");
  position=0;
  for(c=0;c<=MaxNbBits;c++) {
    printf("%d",position);
    if(c!=MaxNbBits) { printf(","); }
    position=position+NbValuesWithNBits[c];
  }
  printf("};\n");
  //number of NCK
  printf("int ValuesWithKBitsOutOfN_Occurence[MaxNbBits+1][MaxNbBits+1]={ //order is [N][K]\n");
  for(c=0;c<=MaxNbBits;c++) { //c=N
  printf("{");
    for(d=0;d<=c;d++) { //d=K
      printf("%d",nCk(c,d));
      if(d!=MaxNbBits) { printf(","); }
    }
    for(d=c+1;d<=MaxNbBits;d++) { printf("0"); if(d!=MaxNbBits) { printf(","); } }
    printf("}");
    if(c!=MaxNbBits) { printf(",\n"); } else { printf("};\n"); }
  }
  //values
  printf("int ValuesWithKBits[MaxNbBits_Power2]={\n");
  position=0;
  for(c=0;c<=MaxNbBits;c++) {
    for(d=0;d<NbValuesWithNBits[c];d++) {
      printf("%d",RulesUsed[position]);
      if((c!=MaxNbBits) || (d!=NbValuesWithNBits[c]-1)) { printf(","); }
      position++;
    }
    printf("\n");
  }
  printf("};\n");

  exit(0);
}

