/* cmp.c compares two files up to first difference */
/* Alain Brobecker - 2009/01/24 */

#include <stdio.h>  /* file functions, etc... */
#include <stdlib.h> /* exit(), malloc() */

#define TRUE 1
#define FALSE 0

void PrintByte(byte) unsigned char byte; { printf("%x%x ",(byte>>4)&15,byte&15); }

/* show bytes from adress[start] to adress[end] */
void HexDump(adress,start,end) unsigned char *adress; int start,end; {
  int c,d,NewLine;
  NewLine=0;
  for(c=start;c<=end;c++) {
    if(NewLine==0) { printf("%6x : ",c); }
    PrintByte(adress[c]);
    NewLine++;
    if((NewLine==16) || (c==end)) {
      NewLine=0;
      /* ascii print */
      for(d=c-15;d<=c;d++) {
        if(adress[d]>=32) { printf("%c",adress[d]); } else { printf("."); }
      }
      printf("\n");
    }
  }  
}

/***************************************************************************/
int main(argc,argv) int argc; char *argv[]; {
  FILE *filep;
  int File1Length,File2Length;
  unsigned char *File1,*File2;
  int DifferenceFound;
  int c,start,end;

  if(argc!=3) { printf("usage: cmp file1 file2\n"); exit(1); }
  
  /* load file1 */
  filep=fopen(argv[1],"rb");
  if(filep==0) { printf("error: %s not found\n",argv[1]); exit(1); }
  fseek(filep,0,2);
  File1Length=(int) ftell(filep);
  fseek(filep,0,0);
  File1=(unsigned char *) malloc((unsigned int) File1Length);
  fread(File1,1,File1Length,filep);
  fclose(filep);
  printf("%s: %d bytes\n",argv[1],File1Length);

  /* load File2 */
  filep=fopen(argv[2],"rb");
  if(filep==0) { printf("error: %s not found\n",argv[2]); exit(1); }
  fseek(filep,0,2);
  File2Length=(int) ftell(filep);
  fseek(filep,0,0);
  File2=(unsigned char *) malloc((unsigned int) File2Length);
  fread(File2,1,File2Length,filep);
  fclose(filep);
  printf("%s: %d bytes\n",argv[2],File2Length);

  /* same length? */
  if(File1Length!=File2Length) {
    if(File1Length>File2Length) { File1Length=File2Length; }
    printf("*** Different lengths, comparing anyway %d bytes\n",File1Length);
  }
  
  /* find first difference */
  DifferenceFound=FALSE;
  c=0;
  while((c<File1Length) && (DifferenceFound==FALSE)) {
    if(File1[c]!=File2[c]) {
      DifferenceFound=TRUE;
    }
    c++;
  }
  c--;
  
  /* no difference? */  
  if(DifferenceFound==FALSE) {
    printf("%d bytes are identical\n",File1Length);
  } else {
  /* if a difference was found, show hex dump */
    printf("*** First difference at offset $%x=%d\n",c,c);
    start=c-16;
    if(start<0) { start=0; }
    end=start+47;
    if(end>=File1Length) { end=File1Length-1; }
    printf("%s contains:\n",argv[1]);
    HexDump(File1,start,end);
    printf("%s contains:\n",argv[2]);
    HexDump(File2,start,end);
  }
  
  exit(0);
}
