/* Read a .BMP file, change color channel and save as .BMP */ 
/* Florian & Alain 2008/12/21 */
#include <stdio.h>  /* file functions, etc... */
#include <stdlib.h> /* exit(), malloc() */

#define Size 1000
#define FALSE 0
#define TRUE 1

/* Global variables */
int HeaderLength;
int ImgPos;
unsigned char *Img;

void PrintWord(int pos,int value) {
  Img[pos]=(unsigned char) value%256; /* value modulo 256 */
  value=value/256; /* next byte (divide by 256 is the same as rotating value by 8 bits) */
  Img[pos+1]=(unsigned char) value;
}

void PrintLongWord(int pos,int value) {
  Img[pos]=(unsigned char) value%256;
  value=value/256;
  Img[pos+1]=(unsigned char) value%256;
  value=value/256;
  Img[pos+2]=(unsigned char) value%256;
  value=value/256;
  Img[pos+3]=(unsigned char) value;
}

int ReadLongWord(int pos) {
    int Value;
    if(pos+3>=HeaderLength) {printf("error: cannot read long word\n"); exit(1);}
    Value=((Img[pos+3]*256+Img[pos+2])*256+Img[pos+1])*256+Img[pos];
    return Value;   
}

int ReadWord(int pos) {
    int Value;
    if(pos+1>=HeaderLength) {printf("error: cannot read word\n"); exit(1);}
    Value=Img[pos+1]*256+Img[pos];
    return Value;   
}

/***************************************************************************/
int main(argc,argv) int argc; char *argv[]; {

  FILE *filep,*fopen(const char *,const char *);
  int pos;
  int x,y,n,divergent;
  double xc,yc,xz,yz,t;
  
  if(argc!=2) { printf("usage: %s outfile\n",argv[0]); exit(1); }
  /* LOAD FILE */
  filep=fopen("header.bmp","rb"); /* rb = read binary */
  if(filep==0) { printf("error: file not found\n"); exit(1); }
  fseek(filep,0,2); /* set the file pointer at the end of file */
  HeaderLength=(int) ftell(filep); /* returns the length of the file */
  fseek(filep,0,0); /* set the file pointer at the start of file */
  Img=(unsigned char *) malloc((unsigned int) HeaderLength+Size*Size*3); /* allocate memory */
  fread(Img,1,HeaderLength,filep); /* load file, 1=size of a byte */
  fclose(filep);

  PrintLongWord(18,Size); /* width */
  PrintLongWord(22,Size); /* height*/
  PrintWord(28,24); /* 24 bpp */
  PrintLongWord(2,HeaderLength+Size*Size*3); /* size */

  pos=54;
  for(y=0;y<Size;y++) {
    for(x=0;x<Size;x++) {
      xc=((double) x - Size/2)/(Size/2);
      yc=((double) y - Size/2)/(Size/2);
      xz=0.0;
      yz=0.0;
      n=0;
      divergent=FALSE;
      while((n<255) && (divergent==FALSE)) {
        t=xz*xz-yz*yz+xc;
        yz=2*xz*yz+yc;
        xz=t;
        if(xz*xz+yz*yz>2) { divergent=TRUE; }
        n++;
      }
      Img[pos]=n; pos++; /* blue */
      Img[pos++]=0; /* green */
      Img[pos++]=0; /* red */
    }
  }
  
  /* save file */
  filep=fopen(argv[1],"wb"); /* write binary */
  fwrite(Img,1,HeaderLength+Size*Size*3,filep);
  fclose(filep);
  exit(0);
}
