// Alain Brobecker - 2010/11/25
// Search sets of pythagorean triples, with sensible values for pupils

#include <stdio.h>  /* file functions, etc... */
#include <stdlib.h> /* exit(), malloc() */
#include <math.h>

#define NbTriplesMax 1000
#define NbEleves 24

char *Names[NbEleves] = {
"ALTINDAGOGLU Alain",
"BAILLEUL Alexandre",
"BONELLE Kvin",
"CHARRON Camille",
"CHARTIER Cyril",
"COQUART Aurlien",
"DEZ Ophlie",
"DONNELEY Valentine",
"DUPONT Julie",
"FIGUEIREDO Dylan",
"FREMAUX Dylan",
"GARCIA Matthieu",
"GARIN Virginie",
"GOIRE Arnaud",
"HUSSENET Sabrina",
"KNOCKAERT Quentin",
"KOENIG Judith",
"KURTZ Cid",
"LEFEBVRE Rmy",
"NEUILLET Amandine",
"PINARD Krystin",
"SOUVANNARATH Lashmie",
"STRA Cyprien",
"VEAU Maxence",
};

/**** From D.E.Knuth's Art of Computer Programming, vol.2, p185 ****/
/**** period of pseudorandom sequence is 2147483646             ****/
#define rndM 2147483647 /* 2^31-1, a Mersenne prime */
#define rndA 48271      /* this does well in the spectral test */
#define rndQ 44488      /* rndM/rndA */
#define rndR 3399       /* rndM%rndA, it's important that rndR<rndQ */
long rndX=1;            /* should be initialised to 0<rndX<rndM */
void rndNext() {
  rndX=rndA*(rndX%rndQ)-rndR*(long)(rndX/rndQ); /* X=AX mod M */
  if(rndX<0) {rndX+=rndM; }
}          

int main() {
  int Triplet[NbTriplesMax][3];
  int NbTriples=0;
  int m,n,p,m2plusn2;
  int c,t0,t1,t2;
  
  printf("m;n;p;m/n\n");
  
  for(m=10;m<99;m++) {
    for(n=m+1;n<100;n++) {
      m2plusn2=m*m+n*n;
      p=(int) sqrt(m2plusn2);
      if((p*p==m2plusn2)
//           && (p<100)
//           && (m%10!=0) && (n%10!=0) && (p%10!=0) 
        ){
        Triplet[NbTriples][0]=m;
        Triplet[NbTriples][1]=n;
        Triplet[NbTriples][2]=p;
        NbTriples++;
        printf("%d;%d;%d;%f\n",m,n,p,(float) m/(float) n);
      }
    }
  }
  
  for(c=0;c<NbEleves;c++) {
    printf("\n%s\n",Names[c]);
    rndNext();
    t0=rndX % NbTriples;
    m=Triplet[t0][0];
    n=Triplet[t0][1];
    p=Triplet[t0][2];
//    printf("BA=%d,%d ; AC=%d,%d ; BC=?\n",m/10,m%10,n/10,n%10);
    printf("BA=%d,%d ; AC=%d,%d ; *** BC=%d,%d *** \n",m/10,m%10,n/10,n%10,p/10,p%10);
    do { rndNext(); t1=rndX % NbTriples; } while(t1==t0);
    m=Triplet[t1][0];
    n=Triplet[t1][1];
    p=Triplet[t1][2];
//    printf("DE=%d,%d ; EF=? ; DF=%d,%d\n",m/10,m%10,p/10,p%10);
    printf("DE=%d,%d ; *** EF=%d,%d *** ; DF=%d,%d\n",m/10,m%10,n/10,n%10,p/10,p%10);
    do { rndNext(); t2=rndX % NbTriples; } while((t2==t1) || (t2==t0));
    m=Triplet[t2][0];
    n=Triplet[t2][1];
    p=Triplet[t2][2];
    printf("GI=%d,%d ; IH=%d,%d ; GH=%d,%d\n\n",m/10,m%10,n/10,n%10,p/10,p%10);
    
    
  }
  
  return(0);
}
